use nom::branch::alt;
use nom::bytes::complete::tag;
use nom::bytes::complete::tag_no_case;
use nom::character::complete::line_ending;
use nom::character::complete::space0;
use nom::combinator::eof;
use nom::combinator::recognize;
use nom::sequence::tuple;

use super::Context;
use crate::parser::error::CustomError;
use crate::parser::error::MyError;
use crate::parser::error::Res;
use crate::parser::exiting::ExitClass;
use crate::parser::greater_element::PropertyDrawer;
use crate::parser::parser_context::ContextElement;
use crate::parser::parser_context::ExitMatcherNode;
use crate::parser::parser_with_context::parser_with_context;
use crate::parser::util::immediate_in_section;
use crate::parser::util::start_of_line;

#[tracing::instrument(ret, level = "debug")]
pub fn property_drawer<'r, 's>(
    context: Context<'r, 's>,
    input: &'s str,
) -> Res<&'s str, PropertyDrawer<'s>> {
    if immediate_in_section(context, "property-drawer") {
        return Err(nom::Err::Error(CustomError::MyError(MyError(
            "Cannot nest objects of the same element",
        ))));
    }
    let (
        remaining,
        (_start_of_line, _leading_whitespace, open_tag, _trailing_whitespace, _line_ending),
    ) = tuple((
        parser_with_context!(start_of_line)(context),
        space0,
        tag_no_case(":PROPERTIES:"),
        space0,
        line_ending,
    ))(input)?;

    let parser_context = context
        .with_additional_node(ContextElement::ConsumeTrailingWhitespace(true))
        .with_additional_node(ContextElement::Context("property-drawer"))
        .with_additional_node(ContextElement::ExitMatcherNode(ExitMatcherNode {
            class: ExitClass::Alpha,
            exit_matcher: &property_drawer_end,
        }));

    // let element_matcher = parser_with_context!(element)(&parser_context);
    // let exit_matcher = parser_with_context!(exit_matcher_parser)(&parser_context);
    // let (remaining, children) = match consumed(many_till(blank_line, exit_matcher))(remaining) {
    //     Ok((remaining, (whitespace, (_children, _exit_contents)))) => (
    //         remaining,
    //         vec![Element::Paragraph(Paragraph::of_text(whitespace))],
    //     ),
    //     Err(_) => {
    //         let (remaining, (children, _exit_contents)) =
    //             many_till(element_matcher, exit_matcher)(remaining)?;
    //         (remaining, children)
    //     }
    // };
    // let (remaining, _end) = drawer_end(&parser_context, remaining)?;

    // let (remaining, _trailing_ws) =
    //     maybe_consume_trailing_whitespace_if_not_exiting(context, remaining)?;
    // let source = get_consumed(input, remaining);

    // Ok((
    //     remaining,
    //     Drawer {
    //         source,
    //         name: drawer_name,
    //         children,
    //     },
    // ))
    todo!()
}

#[tracing::instrument(ret, level = "debug")]
fn property_drawer_end<'r, 's>(context: Context<'r, 's>, input: &'s str) -> Res<&'s str, &'s str> {
    recognize(tuple((
        parser_with_context!(start_of_line)(context),
        space0,
        tag(":end:"),
        space0,
        alt((line_ending, eof)),
    )))(input)
}
